"use strict";
/*
* This program and the accompanying materials are made available under the terms of the *
* Eclipse Public License v2.0 which accompanies this distribution, and is available at *
* https://www.eclipse.org/legal/epl-v20.html                                      *
*                                                                                 *
* SPDX-License-Identifier: EPL-2.0                                                *
*                                                                                 *
* Copyright Contributors to the Zowe Project.                                     *
*                                                                                 *
*/
Object.defineProperty(exports, "__esModule", { value: true });
exports.CallSP = void 0;
const util_1 = require("util");
const ibmdb = require("ibm_db");
const IDB2Parameter_1 = require("./doc/IDB2Parameter");
const SessionValidator_1 = require("./SessionValidator");
const ConnectionString_1 = require("./ConnectionString");
const DB2Constants_1 = require("./DB2Constants");
const DB2Error_1 = require("./DB2Error");
/**
 * Class to handle the invocation of the stored procedures
 * @export
 * @class CallSP
 */
class CallSP {
    /**
     * Call a stored procedure
     * @param {IDB2Session} session DB2 session parameters
     * @param {string} routineName Name of the stored procedure to call
     * @param {IDB2Parameter[]} parameters Parameters to bind to the SQL statement
     * @returns {IDB2Response}
     * @memberof CallSP
     * @static
     */
    static callCommon(session, routineName, parameters) {
        SessionValidator_1.SessionValidator.validate(session);
        const connectionString = ConnectionString_1.ConnectionString.buildFromSession(session);
        const options = {
            fetchMode: DB2Constants_1.DB2Constants.FETCH_MODE_ARRAY,
        };
        const response = {
            success: false,
            results: [],
            failureResponse: undefined,
        };
        const query = `CALL ${routineName}`;
        let result;
        let outVarCount = 0;
        // Count parameters with type OUTPUT or INOUT as they will be returned in the result set.
        if (!util_1.isNullOrUndefined(parameters)) {
            for (const parameter of parameters) {
                if (parameter.ParamType === IDB2Parameter_1.DB2_PARM_INOUT || parameter.ParamType === IDB2Parameter_1.DB2_PARM_OUTPUT) {
                    outVarCount++;
                }
            }
        }
        try {
            const db2 = ibmdb.openSync(connectionString, options);
            // Prepare and execute the statement
            const preparedStatement = db2.prepareSync(query);
            result = preparedStatement.executeSync(parameters);
            // Extract INOUT and OUTPUT parameters
            if (outVarCount !== 0 && Array.isArray(result)) {
                // If there are OUT/INOUT parameters, then ODBC driver returns an array, where
                //   - element with index 0 is the ODBSResult
                //   - element with index 1 is an array of INOUT/OUTPUT parameters
                while (outVarCount > 0) {
                    response.results.push(result[1].shift());
                    outVarCount--;
                }
                result = result[0];
            }
            const data = result.fetchAllSync();
            // Rest is the result set
            if (data.length) {
                response.results.push(data);
            }
            // Retrieve all other result sets
            while (result.moreResultsSync()) {
                response.results.push(result.fetchAllSync());
            }
            response.success = true;
            result.closeSync();
            db2.closeSync();
        }
        catch (err) {
            DB2Error_1.DB2Error.process(err);
        }
        return response;
    }
    /**
     * Call a stored procedure
     * @param {IDB2Session} session session DB2 session parameters
     * @param {string} routineName Name of the stored procedure to call
     * @param {IDB2Parameter} parameters Parameters to bind to the SQL statement
     * @returns {IDB2Response}
     * @memberof CallSP
     * @static
     */
    static call(session, routineName, ...parameters) {
        return CallSP.callCommon(session, routineName, parameters);
    }
}
exports.CallSP = CallSP;
//# sourceMappingURL=CallSP.js.map